/*
 *  Copyright (C) 2018-2021 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <kernel/dpl/DebugP.h>
#include <kernel/dpl/TimerP.h>
#include <kernel/dpl/SemaphoreP.h>
#include <kernel/dpl/ClockP.h>
#include "ti_drivers_config.h"
#include "ti_board_config.h"
#include "FreeRTOS.h"
#include "task.h"
#include <semphr.h>
#include "ti_drivers_open_close.h"
#include "ti_board_open_close.h"


#define MAIN_TASK_PRI  (configMAX_PRIORITIES-1)
#define TIMER8_BASE_ADDR     (0x2480000u)
#define SAMPLE_COUNT 125

#define MAIN_TASK_SIZE (16384U/sizeof(configSTACK_DEPTH_TYPE))
StackType_t gMainTaskStack[MAIN_TASK_SIZE] __attribute__((aligned(32)));

StaticTask_t gMainTaskObj;
TaskHandle_t gMainTask;

StackType_t gEmptyTaskStack[MAIN_TASK_SIZE] __attribute__((aligned(32)));

StaticTask_t gEmptyTaskObj;
TaskHandle_t gEmptyTask;

StackType_t gJitterTaskStack[MAIN_TASK_SIZE] __attribute__((aligned(32)));

StaticTask_t gJitterTaskObj;
TaskHandle_t gJitterTask;

SemaphoreP_Object gBinarySem;

int32_t status = ENET_SOK;


 SemaphoreHandle_t gSem;
 StaticSemaphore_t gSemObj;

 SemaphoreHandle_t gHop2Sem;
 StaticSemaphore_t gHop2SemObj;

 SemaphoreHandle_t gHop3Sem;
 StaticSemaphore_t gHop3SemObj;

 int appMain(void *args);

int jitter(void *args);

int appTimer(void *args);

void freertos_main(void *args)
{
    appMain(NULL); // To enable or disable networking part

    vTaskDelete(NULL);
}

void freertos_empty(void *args)
{
    emptyMain(NULL); // To enable or disable networking part

    vTaskDelete(NULL);
}

int emptyMain(void *args)
{
    //volatile uint32_t StartCounter_empty,endCounter_empty,diffCounter_empty;
    //static uint32_t count =0;

    while(1)
    {
        //StartCounter_empty = CycleCounterP_getCount32();
        xSemaphoreTake( gHop2Sem, portMAX_DELAY); /* wait for wake up from TimeTick */
        xSemaphoreGive( gHop3Sem); /* wake up task */
        //endCounter_empty = CycleCounterP_getCount32();
    }

   return 0;
}

int appMain(void *args)
{
    //volatile uint32_t StartCounter,endCounter,diffCounter;
    //static uint32_t count =0;

    Drivers_open();
    Board_driversOpen();

    while(1)
    {
        //StartCounter = CycleCounterP_getCount32();
        xSemaphoreTake( gSem, portMAX_DELAY); /* wait for wake up from TimeTick */
        xSemaphoreGive( gHop2Sem); /* wake up task */
        //endCounter = CycleCounterP_getCount32();
    }

   return 0;
}

uint32_t gTimerCount = 0;
uint64_t IsrCurrTime_us[SAMPLE_COUNT];

void timerTickIsr(void *args)
{
    //2 From timer ISR give a semaphore to high priority task
    static uint32_t count =0;
    BaseType_t doTaskSwitch = 0;
    uint32_t current_count =0;
    static uint32_t end_count =0;

    //SemaphoreP_post(&gBinarySem);
    current_count = CycleCounterP_getCount32();
    xSemaphoreGiveFromISR( gSem, &doTaskSwitch); /* wake up task */
    portYIELD_FROM_ISR( doTaskSwitch );

    IsrCurrTime_us[count] =( (current_count - end_count) / 800U );
    count++;
    if(count >= SAMPLE_COUNT)
    {
        count = 0;
    }

    end_count = current_count;

}

uint64_t Jitter_CurrTime_us[SAMPLE_COUNT];

int jitter(void *args)
{
    //3. In task measure the difference in task wakeup time. Look for minuimum/maximum.
    static uint64_t count = 0;
    //uint64_t diffTime_us = 0;
    static uint32_t StartCounter = 0;

while(1)
{
    /* wait for 10ms for the semaphore to be post by the peripheral */

    StartCounter = CycleCounterP_getCount32();
    //status = SemaphoreP_pend(&gBinarySem, ClockP_usecToTicks(1*130));
    xSemaphoreTake( gHop3Sem, portMAX_DELAY); /* wait for wake up from TimeTick */

    //const  uint64_t currTime_us  = ClockP_getTimeUsec();
    Jitter_CurrTime_us[count] = ( CycleCounterP_getCount32() - StartCounter ) / 800U;

    count++;

    if(count >= SAMPLE_COUNT)
    {
        count = 0x00;
    }

}
    return 0;

}

void jitter_task(void *args)
{
    jitter(NULL);

    vTaskDelete(NULL);
}

int main(void)
{
    //status = SemaphoreP_constructBinary(&gBinarySem, 0U);

    gSem = xSemaphoreCreateBinaryStatic(&gSemObj);
    gHop2Sem = xSemaphoreCreateBinaryStatic(&gHop2SemObj);
    gHop3Sem = xSemaphoreCreateBinaryStatic(&gHop3SemObj);



    /* init SOC specific modules */
    System_init();
    Board_init();

    /* This task is created at highest priority, it should create more tasks and then delete itself */
    gMainTask = xTaskCreateStatic( freertos_main,   /* Pointer to the function that implements the task. */
                                  "freertos_main", /* Text name for the task.  This is to facilitate debugging only. */
                                  MAIN_TASK_SIZE,  /* Stack depth in units of StackType_t typically uint32_t on 32b CPUs */
                                  NULL,            /* We are not using the task parameter. */
                                  MAIN_TASK_PRI-1,   /* task priority, 0 is lowest priority, configMAX_PRIORITIES-1 is highest */
                                  gMainTaskStack,  /* pointer to stack base */
                                  &gMainTaskObj ); /* pointer to statically allocated task object memory */
    DebugP_assert(gMainTask != NULL);

    /* This task is created at highest priority, it should create more tasks and then delete itself */
    gEmptyTask = xTaskCreateStatic( freertos_empty,   /* Pointer to the function that implements the task. */
                                  "freertos_empty", /* Text name for the task.  This is to facilitate debugging only. */
                                  MAIN_TASK_SIZE,  /* Stack depth in units of StackType_t typically uint32_t on 32b CPUs */
                                  NULL,            /* We are not using the task parameter. */
                                  MAIN_TASK_PRI-1,   /* task priority, 0 is lowest priority, configMAX_PRIORITIES-1 is highest */
                                  gEmptyTaskStack,  /* pointer to stack base */
                                  &gEmptyTaskObj ); /* pointer to statically allocated task object memory */
    DebugP_assert(gMainTask != NULL);

    /* This task is created at highest priority, it should create more tasks and then delete itself */
    gJitterTask = xTaskCreateStatic( jitter_task,   /* Pointer to the function that implements the task. */
                                  "jitter_task", /* Text name for the task.  This is to facilitate debugging only. */
                                  MAIN_TASK_SIZE,  /* Stack depth in units of StackType_t typically uint32_t on 32b CPUs */
                                  NULL,            /* We are not using the task parameter. */
                                  MAIN_TASK_PRI,   /* task priority, 0 is lowest priority, configMAX_PRIORITIES-1 is highest */
                                  gJitterTaskStack,  /* pointer to stack base */
                                  &gJitterTaskObj ); /* pointer to statically allocated task object memory */
    DebugP_assert(gJitterTask != NULL);

    /* Start the scheduler to start the tasks executing. */
    vTaskStartScheduler();

    /* The following line should never be reached because vTaskStartScheduler()
    will only return if there was not enough FreeRTOS heap memory available to
    create the Idle and (if configured) Timer tasks.  Heap management, and
    techniques for trapping heap exhaustion, are described in the book text. */
    DebugP_assertNoLog(0);

    return 0;
}
